#include "config.h"
#include "resoff.h"
#include "boolean.h"

int main()
{
    char *envpath;                    /* environment path name */
    char *envport;
    char filename[256];               /* database file name */
    char lockfile[256];               /* lock file name */
    struct passwd *home;              /* passwd struct for home directory */
    int lfd;                          /* lock file descriptor */
    int fd;                           /* file description of database file */
    int listenfd;
    int connfd;
    int n;
    struct sockaddr_in cliaddr;
    struct sockaddr_in servaddr;
    socklen_t clilen;
    pid_t childpid;
    
    envpath = getenv ("RESOFFDIR");   /* get environment path */
    if ( envpath == NULL ) {  /* module to get database file name from home */
        home = getpwuid(geteuid());
	strcpy(filename, home->pw_dir);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, home->pw_dir);   /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    else {                    /* module to get database file name from env */
        strcpy(filename, envpath);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, envpath);        /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    
    lfd = open (lockfile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
    
    if ( lfd < 0 ) {
        ERR_QUIT("Other server is running or lock is not deleted "
                 "after termination.\n");
    }
    
    fd = open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);

    if ( fd < 0 ) {
        openerrmsg(errno);
        remove_lock();
        exit(ERROR);
    }
    
    initdb(filename);
    
    if ( (listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        socketerrmsg(errno);
        exit(ERROR);
    }
    
    memset (&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = htonl(INADDR_ANY);

    envport = getenv ("RESOFFPORT");   /* get environment port number */
    if ( envport == NULL ) {
        servaddr.sin_port = htons(SERV_PORT);
    }
    else {
        if ( is_all_digit(envport) ) {
            servaddr.sin_port = htons(atoi(envport));
        }
        else {
            ERR_QUIT("Illigal form of  environment RESOFFPORT\n");
        }
    }
    
    if ( (bind (listenfd, (SA *) &servaddr, sizeof (servaddr))) < 0 ) {
        binderrmsg(errno);
        exit(ERROR);
    }
    

    if ( (listen (listenfd, LISTENQ)) < 0 ) {
        ERR_QUIT("listen error\n");
    }
    
    Signal(SIGCHLD, sig_chld);

    for ( ; ; ) {
        clilen = sizeof(cliaddr);
        if ( (connfd = accept(listenfd, (SA *) &cliaddr, &clilen)) < 0) {
            if (errno == EINTR)
                continue;
            else
                ERR_QUIT ("accept");
        }
      
        if ( (childpid = fork()) == 0 ) {
            CHECK (close (listenfd));
            
            if ( (n = readline(connfd, recvline, 30)) != 0 ) {
                resoffserve(connfd);
            }
            
            exit(0);
        }
        CHECK (close (connfd)); /* parent closes connected socket */
    }
}

    

    
